@@ -339,6 +339,8 @@ class VPTHamiltonianOptions:
          "kinetic_terms",
          "coriolis_terms",
          "pseudopotential_terms",
+         "dipole_terms",
+         "dipole_derivatives",
          "undimensionalize_normal_modes",
          "use_numerical_jacobians",
          "eckart_embed_derivatives",
@@ -362,7 +364,8 @@ class VPTHamiltonianOptions:
          "hessian_tolerance",
          "grad_tolerance",
          "freq_tolerance",
-         "g_derivative_threshold"
+         "g_derivative_threshold",
+         "gmatrix_tolerance"
     )
 
     def __init__(self,
@@ -372,6 +375,8 @@ class VPTHamiltonianOptions:
                  kinetic_terms=None,
                  coriolis_terms=None,
                  pseudopotential_terms=None,
+                 dipole_terms=None,
+                 dipole_derivatives=None,
                  undimensionalize_normal_modes=None,
                  use_numerical_jacobians=None,
                  eckart_embed_derivatives=None,
@@ -395,6 +400,7 @@ class VPTHamiltonianOptions:
                  grad_tolerance=None,
                  freq_tolerance=None,
                  g_derivative_threshold=None,
+                 gmatrix_tolerance=None,
                  use_internal_modes=None
                  ):
         """
@@ -456,6 +462,8 @@ class VPTHamiltonianOptions:
             kinetic_terms=kinetic_terms,
             coriolis_terms=coriolis_terms,
             pseudopotential_terms=pseudopotential_terms,
+            dipole_terms=dipole_terms,
+            dipole_derivatives=dipole_derivatives,
             undimensionalize=undimensionalize_normal_modes,
             numerical_jacobians=use_numerical_jacobians,
             use_internal_modes=use_internal_modes,
@@ -479,7 +487,8 @@ class VPTHamiltonianOptions:
             hessian_tolerance=hessian_tolerance,
             grad_tolerance=grad_tolerance,
             freq_tolerance=freq_tolerance,
-            g_derivative_threshold=g_derivative_threshold
+            g_derivative_threshold=g_derivative_threshold,
+            gmatrix_tolerance=gmatrix_tolerance
         )
 
         real_opts = {}
@@ -1032,6 +1041,11 @@ class VPTStateMaker:
 
 class AnneInputHelpers:
 
+    @classmethod
+    def _check_file(cls, no_file):
+        if len(no_file.splitlines()) < 2 and '.' in no_file:
+            raise FileNotFoundError("{} is not a file".format(no_file))
+
     @classmethod
     def get_tensor_idx(cls, line, inds, m):
         bits = line.split()
@@ -1052,13 +1066,16 @@ class AnneInputHelpers:
                     if len(line) > 0:
                         m = cls.get_tensor_idx(line, inds, m)
         else:
+            cls._check_file(block)
             for line in block.splitlines():
                 line = line.strip()
                 if len(line) > 0:
                     m = cls.get_tensor_idx(line, inds, m)
-        m = m + 1
-        n = len(next(iter(inds)))
-        a = np.zeros((m,) * n)
+        if dims is None:
+            m = m + 1
+            n = len(next(iter(inds)))
+            dims = (m,) * n
+        a = np.zeros(dims)
         for i, v in inds.items():
             for p in itertools.permutations(i):
                 a[p] = v
@@ -1083,7 +1100,7 @@ class AnneInputHelpers:
             return None
 
     @classmethod
-    def parse_modes(cls, block, dims=None):
+    def parse_modes(cls, block):
         inds = {}
         m = 0
         freqs = None
@@ -1102,6 +1119,7 @@ class AnneInputHelpers:
                             Linv = cls.parse_modes_line(line, L.shape[1])
                             break
         else:
+            cls._check_file(block)
             for line in block.splitlines():
                 line = line.strip()
                 if len(line) > 0:
@@ -1115,7 +1133,7 @@ class AnneInputHelpers:
         return freqs, L, Linv
 
     @classmethod
-    def parse_coords(cls, block, dims=None):
+    def parse_coords(cls, block):
         coords = []
         if os.path.isfile(block):
             with open(block) as f:
@@ -1124,6 +1142,7 @@ class AnneInputHelpers:
                     if len(line) > 0:
                         coords.append([float(x) for x in line.split()])
         else:
+            cls._check_file(block)
             for line in block.splitlines():
                 line = line.strip()
                 if len(line) > 0:
@@ -1131,36 +1150,106 @@ class AnneInputHelpers:
         return np.array(coords)
 
     @classmethod
-    def parse_atoms(cls, block, dims=None):
+    def parse_atoms(cls, block):
         coords = []
         if os.path.isfile(block):
             with open(block) as f:
                 for line in f:
                     line = line.strip()
                     if len(line) > 0:
-                        coords.append([float(x) for x in line.split()])
+                        try:
+                            coords.extend([int(x) for x in line.split()])
+                        except ValueError:
+                            pass
         else:
+            cls._check_file(block)
             for line in block.splitlines():
                 line = line.strip()
                 if len(line) > 0:
-                    coords.append([float(x) for x in line.split()])
-        return np.array(coords)
+                    try:
+                        coords.extend([int(x) for x in line.split()])
+                    except ValueError:
+                        pass
+        return [AtomData[x]["Symbol"] for x in coords]
 
     @classmethod
-    def renormalize_modes(cls, freqs, modes, inv):
+    def parse_zmatrix(cls, block):
+        _ = 10000
+        zmat = [[0, _, _, _]]
+        n = 1
+        if os.path.isfile(block):
+            with open(block) as f:
+                for line in f:
+                    line = line.strip()
+                    if len(line) > 0:
+                        split = line.split()
+                        if len(split) == 4:
+                            split = split[:3]
+                        zmat.append([n] + [int(x)-1 for x in split] + [_]*(3-len(split)))
+                        n += 1
+                    elif len(zmat) > 1:
+                        break
+        else:
+            cls._check_file(block)
+            for line in block.splitlines():
+                line = line.strip()
+                if len(line) > 0:
+                    split = line.split()
+                    if len(split) == 4:
+                        split = split[:3]
+                    zmat.append([n] + [int(x) - 1 for x in split] + [_] * (3 - len(split)))
+                    n += 1
+                elif len(zmat) > 1:
+                    break
+        return np.array(zmat)
+
+    @classmethod
+    def standard_sorting(cls, zmat):
+        """
+        converts from [r1, r2, r3, ..., a1, a2, ..., t1, t2, ...] coords
+        to standard zmat coords
+        :param zmat:
+        :type zmat:
+        :return:
+        :rtype:
+        """
+        nats = len(zmat)
+        ncoords = 3*nats - 6
+        if nats < 4:
+            return None
+        else:
+            r_coords = [0, 1, 3]
+            a_coords = [2, 4]
+            t_coords = [5]
+            extra = np.arange(6, ncoords)
+            r_coords += extra[::4].tolist()
+            a_coords += extra[1::4].tolist()
+            t_coords += extra[2::4].tolist()
+            return np.argsort(np.concatenate([r_coords, a_coords, t_coords]))
+
+    @classmethod
+    def renormalize_modes(cls, freqs, modes, inv, sorting=None):
         # modes = (modes * np.sqrt(freqs)[:, np.newaxis]).T
         # inv = inv#.T / np.sqrt(freqs)[:, np.newaxis]
         G = np.dot(modes, modes.T)
         F = np.dot(np.dot(inv.T, np.diag(freqs ** 2)), inv)
+        if sorting is not None:
+            G = G[np.ix_(sorting, sorting)]
+            # print(np.round(F*219465))
+            F = F[np.ix_(sorting, sorting)]
         freq, modes = scipy.linalg.eigh(F, G, type=2)
-        return freq, modes, np.linalg.inv(modes)
+        return np.sqrt(freq), modes, np.linalg.inv(modes)
 
     @classmethod
-    def rerotate_force_field(cls, old_inv, new_modes, old_field):
+    def rerotate_force_field(cls, old_inv, new_modes, old_field, sorting=None):
         mid_field = []
         for f in old_field:
             for i in range(f.ndim):
                 f = np.tensordot(old_inv, f, axes=[1, -1])
+                # print(np.round(f*219465))
+            if sorting is not None:
+                f = f[np.ix_(*(sorting,)*f.ndim)]
+                # print(np.round(f*219465))
             mid_field.append(f)
         new_field = []
         for f in mid_field:
@@ -1170,12 +1259,13 @@ class AnneInputHelpers:
         return new_field, mid_field
 
     @classmethod
-    def reexpress_normal_modes(cls, base_modes, old_field):
-        freq, matrix, inv = cls.renormalize_modes(*base_modes)
+    def reexpress_normal_modes(cls, base_modes, old_field, sorting=None):
+        freq, matrix, inv = cls.renormalize_modes(*base_modes, sorting=sorting)
         potential_terms = cls.rerotate_force_field(
-            base_modes[2],
-            inv.T,  # we divide by the sqrt of the frequencies to get the units to work
-            old_field
+            base_modes[1],
+            matrix.T,#/np.sqrt(freq)[np.newaxis, :],  # we divide by the sqrt of the frequencies to get the units to work
+            old_field,
+            sorting=sorting
             # [
             #     np.diag(freq),  # in the file I was given the frequencies were in Hartree
             #     cubics * helpers.convert("Wavenumbers", "Hartrees"),
@@ -1187,5 +1277,108 @@ class AnneInputHelpers:
     convert = UnitsData.convert
     @staticmethod
     def mass(atom):
-        return AtomData[atom]["Mass"]
+        return AtomData[atom]["Mass"] * UnitsData.convert("AtomicMassUnits", "AtomicUnitOfMass")
+
+
+    @classmethod
+    def run_anne_job(cls, base_dir,
+                     states=2,
+                     calculate_intensities=False,
+                     return_analyzer=False,
+                     modes_file='nm_int.dat',
+                     atoms_file='atom.dat',
+                     coords_file='cart_ref.dat',
+                     zmat_file='z_mat.dat',
+                     potential_files=('cub.dat', 'quart.dat'),
+                     energy_units="Wavenumbers",
+                     **opts
+                     ):
+        from .Analyzer import VPTAnalyzer
+
+        og_dir = os.getcwd()
+        try:
+            if base_dir is not None:
+                os.chdir(base_dir)
+
+            conv = cls.convert(energy_units, "Hartrees")
+            base_freqs, base_mat, base_inv = cls.parse_modes(modes_file)
+            base_freqs *= conv
+            potential = [cls.parse_tensor(f) * conv for f in potential_files]
+            atoms = cls.parse_atoms(atoms_file)
+            coords = cls.parse_coords(coords_file)
+            zmat = cls.parse_zmatrix(zmat_file)
+            sorting = cls.standard_sorting(zmat)  # we need to re-sort our internal coordinates
+
+            (freq, matrix, inv), potential_terms = cls.reexpress_normal_modes(
+                (base_freqs, base_mat, base_inv),
+                [np.diag(base_freqs)] + potential,
+                sorting=sorting
+            )
+
+            # raise Exception(np.diag(potential_terms[0]) * UnitsData.convert("Hartrees", "Wavenumbers"), freq*UnitsData.convert("Hartrees", "Wavenumbers"))
+            if return_analyzer:
+                runner = VPTAnalyzer.run_VPT
+            else:
+                runner = VPTRunner.run_simple
+
+            res = runner(
+                [atoms, coords],
+                states,
+                modes={
+                    "freqs": freq,
+                    "matrix": matrix,
+                    "inverse": inv
+                },
+                potential_terms=potential_terms,
+                internals=zmat,
+                calculate_intensities=calculate_intensities,
+                **opts
+            )
+            if return_analyzer:
+                res.print_output_tables(print_intensities=calculate_intensities, print_energies=not calculate_intensities)
+
+        finally:
+            os.chdir(og_dir)
+
+        return res
+
+    @classmethod
+    def get_internal_expansion(cls, fchk, internals, states=2, **opts):
+        test_runner, _ = VPTRunner.construct(
+            fchk,
+            states,
+            internals=internals,
+            logger=False
+        )
+        test_freqs = test_runner.hamiltonian.modes.freqs
+        test_V = test_runner.hamiltonian.V_terms
+        test_modes = test_V.internal_L_matrix
+        test_inver = test_V.internal_L_inverse
+        return {
+            "runner":test_runner,
+            "freqs":test_freqs,
+            "potential":test_V,
+            "modes":[test_modes, test_inver],
+            "states":states,
+            "fchk":fchk,
+            "zmatrix":internals
+        }
+
+    @classmethod
+    def run_internal_expansion(cls, expansion_data, calculate_intensities=False, **opts):
+        test_V = expansion_data['potential']
+        return VPTRunner.run_simple(
+            expansion_data['fchk'],
+            expansion_data['states'],
+            modes={
+                "freqs": expansion_data['freqs'],
+                "matrix": expansion_data['modes'][0],
+                "inverse": expansion_data['modes'][1]
+            },
+            internals=expansion_data['zmatrix'],
+            potential_terms=[test_V[0], test_V[1], test_V[2]],
+            undimensionalize_normal_modes=False,
+            calculate_intensities=calculate_intensities
+        )
+
 VPTRunner.helpers = AnneInputHelpers
\ No newline at end of file
